import unittest
import os
from unittest import mock

import gostcrypto
from gostcrypto.utils import bytearray_to_int
from gostcrypto.utils import int_to_bytearray

TEST_CURVE = {
    'id-tc26-gost-3410-2012-256-paramSetTest':dict(
        p=bytearray_to_int(bytearray.fromhex('8000000000000000000000000000000000000000000000000000000000000431')),
        a=0x07,
        b=bytearray_to_int(bytearray.fromhex('5fbff498aa938ce739b8e022fbafef40563f6e6a3472fc2a514c0ce9dae23b7e')),
        m=bytearray_to_int(bytearray.fromhex('8000000000000000000000000000000150fe8a1892976154c59cfc193accf5b3')),
        q=bytearray_to_int(bytearray.fromhex('8000000000000000000000000000000150fe8a1892976154c59cfc193accf5b3')),
        x=0x02,
        y=bytearray_to_int(bytearray.fromhex('08e2a8a0e65147d4bd6316030e16d19c85c97f0a9ca267122b96abbcea7e8fc8'))
    ),
    'id-tc26-gost-3410-2012-512-paramSetTest':dict(
        p=bytearray_to_int(bytearray.fromhex('4531acd1fe0023c7550d267b6b2fee80922b14b2ffb90f04d4eb7c09b5d2d15df1d852741af4704a0458047e80e4546d35b8336fac224dd81664bbf528be6373')),
        a=0x07,
        b=bytearray_to_int(bytearray.fromhex('1cff0806a31116da29d8cfa54e57eb748bc5f377e49400fdd788b649eca1ac4361834013b2ad7322480a89ca58e0cf74bc9e540c2add6897fad0a3084f302adc')),
        m=bytearray_to_int(bytearray.fromhex('4531acd1fe0023c7550d267b6b2fee80922b14b2ffb90f04d4eb7c09b5d2d15da82f2d7ecb1dbac719905c5eecc423f1d86e25edbe23c595d644aaf187e6e6df')),
        q=bytearray_to_int(bytearray.fromhex('4531acd1fe0023c7550d267b6b2fee80922b14b2ffb90f04d4eb7c09b5d2d15da82f2d7ecb1dbac719905c5eecc423f1d86e25edbe23c595d644aaf187e6e6df')),
        x=bytearray_to_int(bytearray.fromhex('24d19cc64572ee30f396bf6ebbfd7a6c5213b3b3d7057cc825f91093a68cd762fd60611262cd838dc6b60aa7eee804e28bc849977fac33b4b530f1b120248a9a')),
        y=bytearray_to_int(bytearray.fromhex('2bb312a43bd2ce6e0d020613c857acddcfbf061e91e5f2c3f32447c259f39b2c83ab156d77f1496bf7eb3351e1ee4e43dc1a18b91b24640b6dbb92cb1add371e'))
    ),
    'id-tc26-gost-3410-2012-256-paramSetTestEdvardsA': dict(
        p=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
            0x97
        ])),
        e=0x01,
        d=bytearray_to_int(bytearray([
            0x06, 0x05, 0xf6, 0xb7, 0xc1, 0x83, 0xfa, 0x81,
            0x57, 0x8b, 0xc3, 0x9c, 0xfa, 0xd5, 0x18, 0x13,
            0x2b, 0x9d, 0xf6, 0x28, 0x97, 0x00, 0x9a, 0xf7,
            0xe5, 0x22, 0xc3, 0x2d, 0x6d, 0xc7, 0xbf, 0xfb
        ])),
        m=bytearray_to_int(bytearray([
            0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x3f, 0x63, 0x37, 0x7f, 0x21, 0xed, 0x98,
            0xd7, 0x04, 0x56, 0xbd, 0x55, 0xb0, 0xd8, 0x31,
            0x9c
        ])),
        q=bytearray_to_int(bytearray([
            0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x0f, 0xd8, 0xcd, 0xdf, 0xc8, 0x7b, 0x66, 0x35,
            0xc1, 0x15, 0xaf, 0x55, 0x6c, 0x36, 0x0c, 0x67
        ])),
        u=0x0d,
        v=bytearray_to_int(bytearray([
            0x60, 0xca, 0x1e, 0x32, 0xaa, 0x47, 0x5b, 0x34,
            0x84, 0x88, 0xc3, 0x8f, 0xab, 0x07, 0x64, 0x9c,
            0xe7, 0xef, 0x8d, 0xbe, 0x87, 0xf2, 0x2e, 0x81,
            0xf9, 0x2b, 0x25, 0x92, 0xdb, 0xa3, 0x00, 0xe7
        ])),
    ),
    'id-tc26-gost-3410-2012-256-paramSetTestEdvardsB': dict(
        p=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
            0x97
        ])),
        a=bytearray_to_int(bytearray([
            0x00, 0xc2, 0x17, 0x3f, 0x15, 0x13, 0x98, 0x16,
            0x73, 0xaf, 0x48, 0x92, 0xc2, 0x30, 0x35, 0xa2,
            0x7c, 0xe2, 0x5e, 0x20, 0x13, 0xbf, 0x95, 0xaa,
            0x33, 0xb2, 0x2c, 0x65, 0x6f, 0x27, 0x7e, 0x73,
            0x35
        ])),
        b=bytearray_to_int(bytearray([
            0x29, 0x5f, 0x9b, 0xae, 0x74, 0x28, 0xed, 0x9c,
            0xcc, 0x20, 0xe7, 0xc3, 0x59, 0xa9, 0xd4, 0x1a,
            0x22, 0xfc, 0xcd, 0x91, 0x08, 0xe1, 0x7b, 0xf7,
            0xba, 0x93, 0x37, 0xa6, 0xf8, 0xae, 0x95, 0x13
        ])),
        e=0x01,
        d=bytearray_to_int(bytearray([
            0x06, 0x05, 0xf6, 0xb7, 0xc1, 0x83, 0xfa, 0x81,
            0x57, 0x8b, 0xc3, 0x9c, 0xfa, 0xd5, 0x18, 0x13,
            0x2b, 0x9d, 0xf6, 0x28, 0x97, 0x00, 0x9a, 0xf7,
            0xe5, 0x22, 0xc3, 0x2d, 0x6d, 0xc7, 0xbf, 0xfb
        ])),
        m=bytearray_to_int(bytearray([
            0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x3f, 0x63, 0x37, 0x7f, 0x21, 0xed, 0x98,
            0xd7, 0x04, 0x56, 0xbd, 0x55, 0xb0, 0xd8, 0x31,
            0x9c
        ])),
        q=bytearray_to_int(bytearray([
            0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x0f, 0xd8, 0xcd, 0xdf, 0xc8, 0x7b, 0x66, 0x35,
            0xc1, 0x15, 0xaf, 0x55, 0x6c, 0x36, 0x0c, 0x67
        ])),
        x=bytearray_to_int(bytearray([
            0x00, 0x91, 0xe3, 0x84, 0x43, 0xa5, 0xe8, 0x2c,
            0x0d, 0x88, 0x09, 0x23, 0x42, 0x57, 0x12, 0xb2,
            0xbb, 0x65, 0x8b, 0x91, 0x96, 0x93, 0x2e, 0x02,
            0xc7, 0x8b, 0x25, 0x82, 0xfe, 0x74, 0x2d, 0xaa,
            0x28
        ])),
        y=bytearray_to_int(bytearray([
            0x32, 0x87, 0x94, 0x23, 0xab, 0x1a, 0x03, 0x75,
            0x89, 0x57, 0x86, 0xc4, 0xbb, 0x46, 0xe9, 0x56,
            0x5f, 0xde, 0x0b, 0x53, 0x44, 0x76, 0x67, 0x40,
            0xaf, 0x26, 0x8a, 0xdb, 0x32, 0x32, 0x2e, 0x5c
        ])),
        u=0x0d,
        v=bytearray_to_int(bytearray([
            0x60, 0xca, 0x1e, 0x32, 0xaa, 0x47, 0x5b, 0x34,
            0x84, 0x88, 0xc3, 0x8f, 0xab, 0x07, 0x64, 0x9c,
            0xe7, 0xef, 0x8d, 0xbe, 0x87, 0xf2, 0x2e, 0x81,
            0xf9, 0x2b, 0x25, 0x92, 0xdb, 0xa3, 0x00, 0xe7
        ])),
    )
}
TEST_PRIVATE_KEY_256 = bytearray.fromhex('7a929ade789bb9be10ed359dd39a72c11b60961f49397eee1d19ce9891ec3b28')
TEST_DIGEST_256 = bytearray.fromhex('2dfbc1b372d89a1188c09c52e0eec61fce52032ab1022e8e67ece6672b043ee5')
TEST_RANDOM_256 = bytearray.fromhex('77105c9b20bcd3122823c8cf6fcc7b956de33814e95b7fe64fed924594dceab3')
TEST_SIGNATURE_256 = bytearray.fromhex('41aa28d2f1ab148280cd9ed56feda41974053554a42767b83ad043fd39dc049301456c64ba4642a1653c235a98a60249bcd6d3f746b631df928014f6c5bf9c40')
TEST_SIGNATURE_256_ZERO = bytearray.fromhex('00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000')
TEST_PUBLIC_KEY_256 = bytearray.fromhex('7f2b49e270db6d90d8595bec458b50c58585ba1d4e9b788f6689dbd8e56fd80b26f1b489d6701dd185c8413a977b3cbbaf64d1c593d26627dffb101a87ff77da')

TEST_PRIVATE_KEY_512 = bytearray.fromhex('0ba6048aadae241ba40936d47756d7c93091a0e8514669700ee7508e508b102072e8123b2200a0563322dad2827e2714a2636b7bfd18aadfc62967821fa18dd4')
TEST_DIGEST_512 = bytearray.fromhex('3754f3cfacc9e0615c4f4a7c4d8dab531b09b6f9c170c533a71d147035b0c5917184ee536593f4414339976c647c5d5a407adedb1d560c4fc6777d2972075b8c')
TEST_RANDOM_512 = bytearray.fromhex('0359e7f4b1410feacc570456c6801496946312120b39d019d455986e364f365886748ed7a44b3e794434006011842286212273a6d14cf70ea3af71bb1ae679f1')
TEST_SIGNATURE_512 = bytearray.fromhex('2f86fa60a081091a23dd795e1e3c689ee512a3c82ee0dcc2643c78eea8fcacd35492558486b20f1c9ec197c90699850260c93bcbcd9c5c3317e19344e173ae361081b394696ffe8e6585e7a9362d26b6325f56778aadbc081c0bfbe933d52ff5823ce288e8c4f362526080df7f70ce406a6eeb1f56919cb92a9853bde73e5b4a')
TEST_PUBLIC_KEY_512 = bytearray.fromhex('115dc5bc96760c7b48598d8ab9e740d4c4a85a65be33c1815b5c320c854621dd5a515856d13314af69bc5b924c8b4ddff75c45415c1d9dd9dd33612cd530efe137c7c90cd40b0f5621dc3ac1b751cfa0e2634fa0503b3d52639f5d7fb72afd61ea199441d943ffe7f0c70a2759a3cdb84c114e1f9339fdf27f35eca93677beec')

count_urandom = 0

def os_urandom(value):
    add = bytearray([0xff, 0xff])
    global count_urandom
    count_urandom = count_urandom + 1
    if count_urandom == 1:
        return TEST_RANDOM_512 + add
    else:
        return TEST_RANDOM_512

class TestMODE256(unittest.TestCase):

    def test_init_raises(self):
        with self.assertRaises(ValueError) as context:
            test_sign = gostcrypto.gostsignature.GOST34102012(None)
        self.assertTrue('unsupported signature mode' in str(context.exception))

    def test_init_raises_1(self):
        with self.assertRaises(ValueError) as context:
            test_sign = gostcrypto.gostsignature.GOST34102012(gostcrypto.gostsignature.MODE_512,
                                                              p=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['p'],
                                                              a=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['a'],
                                                              b=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['b'],
                                                              e = None,
                                                              d = None,
                                                              m=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['p'],
                                                              q=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['q'],
                                                              x=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['x'],
                                                              y=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['y'],
                                                              u = None,
                                                              v = None,)
        self.assertTrue('invalid parameters of the elliptic curve' in str(context.exception))

    def test_init_raises_2(self):
        with self.assertRaises(ValueError) as context:
            test_sign = gostcrypto.gostsignature.GOST34102012(gostcrypto.gostsignature.MODE_256,
                                                              p=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['p'],
                                                              a=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['a'],
                                                              b=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['b'],
                                                              e = None,
                                                              d = None,
                                                              m=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['m'],
                                                              q=0x41,
                                                              x=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['x'],
                                                              y=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['y'],
                                                              u = None,
                                                              v = None,)
        self.assertTrue('invalid parameters of the elliptic curve' in str(context.exception))

    def test_init_raises_3(self):
        with self.assertRaises(ValueError) as context:
            test_sign = gostcrypto.gostsignature.GOST34102012(gostcrypto.gostsignature.MODE_256,
                                                              p=0x02,
                                                              a=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['a'],
                                                              b=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['b'],
                                                              e = None,
                                                              d = None,
                                                              m=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['m'],
                                                              q=0x70,
                                                              x=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['x'],
                                                              y=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['y'],
                                                              u = None,
                                                              v = None,)
        self.assertTrue('invalid parameters of the elliptic curve' in str(context.exception))

    def test_init_raises_4(self):
        with self.assertRaises(ValueError) as context:
            test_sign = gostcrypto.gostsignature.GOST34102012(gostcrypto.gostsignature.MODE_512,
                                                              p=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['p'],
                                                              a=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['a'],
                                                              b=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['b'],
                                                              e = None,
                                                              d = None,
                                                              m=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['m'],
                                                              q=0x02,
                                                              x=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['x'],
                                                              y=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['y'],
                                                              u = None,
                                                              v = None,)
        self.assertTrue('invalid parameters of the elliptic curve' in str(context.exception))

    def test_init_raises_5(self):
        with self.assertRaises(ValueError) as context:
            test_sign = gostcrypto.gostsignature.GOST34102012(gostcrypto.gostsignature.MODE_512,
                                                              p=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['p'],
                                                              a=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['a'],
                                                              b=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['b'],
                                                              e = None,
                                                              d = None,
                                                              m=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['m'],
                                                              q=0x41,
                                                              x=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['x'],
                                                              y=TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest']['y'],
                                                              u = None,
                                                              v = None,)
        self.assertTrue('invalid parameters of the elliptic curve' in str(context.exception))

    def test_init_raises_6(self):
        with self.assertRaises(ValueError) as context:
            test_sign = gostcrypto.gostsignature.GOST34102012(gostcrypto.gostsignature.MODE_256,
                                                              p=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['p'],
                                                              a=0x07,
                                                              b=0x0e,
                                                              e = None,
                                                              d = None,
                                                              m=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['m'],
                                                              q=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['q'],
                                                              x=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['x'],
                                                              y=TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest']['y'],
                                                              u = None,
                                                              v = None,)
        self.assertTrue('invalid parameters of the elliptic curve' in str(context.exception))

    def test_sign_256(self):
        test_sign = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_256, TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest'])
        self.assertEqual(test_sign.sign(TEST_PRIVATE_KEY_256, TEST_DIGEST_256, TEST_RANDOM_256), TEST_SIGNATURE_256)

    def test_sign_edvards_a(self):
        signature = bytearray.fromhex('01b69df1e54b083516a398ee54f6975b213c384b39020bfa983766b7a458a23e370e36ebb493adab5abc69a44f835cbd9bb27db5700bde53f989d2afeb3979e7')
        test_sign = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_256, TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTestEdvardsA'])
        self.assertEqual(test_sign.sign(TEST_PRIVATE_KEY_256, TEST_DIGEST_256, TEST_RANDOM_256), signature)

    def test_sign_edvards_b(self):
        signature = bytearray.fromhex('01b69df1e54b083516a398ee54f6975b213c384b39020bfa983766b7a458a23e370e36ebb493adab5abc69a44f835cbd9bb27db5700bde53f989d2afeb3979e7')
        test_sign = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_256, TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTestEdvardsB'])
        self.assertEqual(test_sign.sign(TEST_PRIVATE_KEY_256, TEST_DIGEST_256, TEST_RANDOM_256), signature)

    def test_sign_raises(self):
        test_sign = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_256, TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest'])
        with self.assertRaises(ValueError) as context:
            self.assertEqual(test_sign.sign(TEST_PRIVATE_KEY_256[:20], TEST_DIGEST_256, TEST_RANDOM_256), TEST_SIGNATURE_256)
        self.assertTrue('invalid private key size' in str(context.exception))
        with self.assertRaises(ValueError) as context:
            test_sign.sign(None, TEST_DIGEST_256, TEST_RANDOM_256)
        self.assertTrue('invalid private key value' in str(context.exception))
        with self.assertRaises(ValueError) as context:
            test_sign.sign(TEST_PRIVATE_KEY_256, TEST_DIGEST_256, TEST_RANDOM_256[:20])
        self.assertTrue('invalid random value size' in str(context.exception))

    def test_verify_256(self):
        test_sign = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_256, TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest'])
        self.assertEqual(test_sign.verify(TEST_PUBLIC_KEY_256, TEST_DIGEST_256, TEST_SIGNATURE_256), True)
        self.assertEqual(test_sign.verify(TEST_PUBLIC_KEY_256, TEST_DIGEST_256, TEST_SIGNATURE_256_ZERO), False)

    def test_verify_raises(self):
        test_sign = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_256, TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest'])
        with self.assertRaises(ValueError) as context:
            test_sign.verify(TEST_PUBLIC_KEY_256, TEST_DIGEST_256, TEST_SIGNATURE_256[:20])
        self.assertTrue('invalid signature size' in str(context.exception))

    def test_public_key_generate_256(self):
        test_sign = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_256, TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest'])
        self.assertEqual(test_sign.public_key_generate(TEST_PRIVATE_KEY_256), TEST_PUBLIC_KEY_256)

    def test_public_key_generate_raises(self):
        test_sign = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_256, TEST_CURVE['id-tc26-gost-3410-2012-256-paramSetTest'])
        with self.assertRaises(ValueError) as context:
            test_sign.public_key_generate(TEST_PRIVATE_KEY_256[:20])
        self.assertTrue('invalid private key size' in str(context.exception))
        with self.assertRaises(ValueError) as context:
            test_sign.public_key_generate(None)
        self.assertTrue('invalid private key value' in str(context.exception))

class TestMODE512(unittest.TestCase):

    def test_sign_512(self):
        test_sign = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_512, TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest'])
        self.assertEqual(test_sign.sign(TEST_PRIVATE_KEY_512, TEST_DIGEST_512, TEST_RANDOM_512), TEST_SIGNATURE_512)

    def test_verify_512(self):
        test_sign = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_512, TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest'])
        self.assertEqual(test_sign.verify(TEST_PUBLIC_KEY_512, TEST_DIGEST_512, TEST_SIGNATURE_512), True)

    def test_public_key_generate_512(self):
        test_sign = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_512, TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest'])
        self.assertEqual(test_sign.public_key_generate(TEST_PRIVATE_KEY_512), TEST_PUBLIC_KEY_512)

    def test_sign_urandom(self):
        test_sign = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_512, TEST_CURVE['id-tc26-gost-3410-2012-512-paramSetTest'])
        with mock.patch('os.urandom', os_urandom):
            test_result = test_sign.sign(TEST_PRIVATE_KEY_512, TEST_DIGEST_512)
        self.assertEqual(test_result, TEST_SIGNATURE_512)


"""The module that implements pseudo-random sequence generation in accordance with
   R 1323565.1.006-2017.

   Author: Evgeny Drobotun (c) 2020
   License: MIT

   Usage:

       import gostcrypto

       random_obj = gostcrypto.gostrandom.new(32)
       result = random_obj.random()

"""
import os
from sys import exit as sys_exit

from gostcrypto.gosthash import GOST34112012
from gostcrypto.utils import bytearray_to_int
from gostcrypto.utils import int_to_bytearray
from gostcrypto.utils import zero_fill

__all__ = ['R132356510062017', 'new',
           'SIZE_S_384', 'SIZE_S_320', 'SIZE_S_256']

SIZE_S_384 = 48 #The size of the initial filling (seed) is 384 bits
SIZE_S_320 = 40 #The size of the initial filling (seed) is 320 bits
SIZE_S_256 = 32 #The size of the initial filling (seed) is 256 bits

_SIZE_M = 64
_SIZE_H = 64

def new(rand_size, rand_k=None, size_s=SIZE_S_384):
    """Creates a new pseudo-random sequence generation object and returns it.

       Args:
          :rand_size: Size of the generated random variable (in bytes).
          :rand_k: Initial filling (seed). If this argument is not passed to the function,
             the 'os.urandom' function is used to generate the initial filling.
          :size_s: Size of the initial filling (in bytes).

       Return:
          New object of generate random value.

       Exception:
          ValueError('invalid seed value size') - in case of invalid size of  initial filling.
    """
    try:
        return R132356510062017(rand_size, rand_k, size_s)
    except ValueError as err:
        print(err)
        sys_exit()

class R132356510062017:
    """Class that implements pseudo-random sequence generation in accordance with
       R 1323565.1.006-2017.

       Methods:
          :random(): generating the next value from a pseudo-random sequence.
          :reset(): resetting the counter and setting a new initial filling.
          :clear(): clearing the counter value.
    """
    def __init__(self, rand_size, rand_k, size_s):
        #Initialize the random object
        self._size_s = size_s
        if rand_k is None:
            self._rand_u = os.urandom(self._size_s) + b'\x00' * (_SIZE_M - self._size_s - 1)
            self._rand_u = bytearray(self._rand_u)
        else:
            if len(rand_k) != self._size_s:
                raise ValueError('ValueError: invalid seed value size')
            self._rand_u = rand_k + b'\x00' * (_SIZE_M - self._size_s - 1)
            self._rand_u = bytearray(self._rand_u)
        self._q = rand_size // _SIZE_H
        self._r = rand_size % _SIZE_H
        self._limit = 2 ** (_SIZE_M - self._size_s)
        self._hash_obj = GOST34112012('streebog512')

    def __del__(self):
        #Delete the random object
        self.clear()

    def _inc_rand_u(self):
        #Increasing the value of the generation counter
        self._rand_u = bytearray_to_int(self._rand_u) + 1 % (2 ** (_SIZE_M - 1))
        self._rand_u = int_to_bytearray(self._rand_u, _SIZE_M - 1)

    def random(self):
        """Generating the next value from a pseudo-random sequence.

           Return:
             New random value.

           Exception:
              ValueError ('exceeded the limit value of the counter') - when the counter limit
                 is exceeded.
              ValueError ('the seed value is zero') - if the seed value is zero.
        """
        if bytearray_to_int(self._rand_u[:self._size_s]) == 0:
            raise ValueError('ValueError: the seed value is zero')
        i = self._q
        result = bytearray(0)
        while i > 0:
            if bytearray_to_int(self._rand_u[self._size_s::]) >= self._limit:
                self._rand_u = zero_fill(len(self._rand_u))
                raise ValueError('ValueError: exceeded the limit value of the counter')
            self._inc_rand_u()
            self._hash_obj.update(self._rand_u)
            rand_c = self._hash_obj.digest()
            self._hash_obj.reset()
            result = result + rand_c
            i = i - 1
        if self._r != 0:
            if bytearray_to_int(self._rand_u[self._size_s::]) >= self._limit:
                self._rand_u = zero_fill(len(self._rand_u))
                raise ValueError('ValueError: exceeded the limit value of the counter')
            self._inc_rand_u()
            self._hash_obj.update(self._rand_u)
            rand_c = self._hash_obj.digest()
            self._hash_obj.reset()
            result = result + rand_c[_SIZE_H - self._r:_SIZE_H:]
        return result

    def reset(self, rand_k=None):
        """Resetting the counter and setting a new initial filling.

           Args:
              :rand_k: New initial filling (seed). If this argument is not passed to the function,
                 the 'os.urandom' function is used to generate the initial filling.

           Exception:
              ValueError('invalid seed value size') - in case of invalid size of  initial filling.
        """
        if rand_k is None:
            self._rand_u = os.urandom(self._size_s) + b'\x00' * (_SIZE_M - self._size_s - 1)
            self._rand_u = bytearray(self._rand_u)
        else:
            if len(rand_k) != self._size_s:
                raise ValueError('ValueError: invalid seed value size')
            self._rand_u = rand_k + b'\x00' * (_SIZE_M - self._size_s - 1)
            self._rand_u = bytearray(self._rand_u)
        self._hash_obj.reset()

    def clear(self):
        """Clearing the counter value.
        """
        self._rand_u = zero_fill(len(self._rand_u))



import unittest
import gostcrypto
from unittest import mock

def os_urandom(value):
    return TEST_SEED

TEST_SEED_ZERO = bytearray([
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
])

TEST_SEED = bytearray([
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
])

TEST_RAND = bytearray([
    0xa8, 0xe2, 0xf9, 0x00, 0xdd, 0x4d, 0x7e, 0x24,
    0x5f, 0x09, 0x75, 0x3d, 0x01, 0xe8, 0x75, 0xfc,
    0x38, 0xf1, 0x4f, 0xf5, 0x25, 0x4c, 0x94, 0xea,
    0xdb, 0x45, 0x1e, 0x4a, 0xb6, 0x03, 0xb1, 0x47,
])

TEST_RAND_LONG = bytearray([
    0x66, 0xc5, 0xb4, 0xf0, 0x4f, 0xfa, 0xc3, 0x59,
    0xd6, 0x50, 0xd0, 0xff, 0x6c, 0xa8, 0x88, 0x73,
    0x69, 0x79, 0x79, 0xeb, 0x9c, 0xc6, 0xc6, 0x3d,
    0x3e, 0xab, 0x26, 0x62, 0xf7, 0xd4, 0x68, 0x75,
    0xa8, 0xe2, 0xf9, 0x00, 0xdd, 0x4d, 0x7e, 0x24,
    0x5f, 0x09, 0x75, 0x3d, 0x01, 0xe8, 0x75, 0xfc,
    0x38, 0xf1, 0x4f, 0xf5, 0x25, 0x4c, 0x94, 0xea,
    0xdb, 0x45, 0x1e, 0x4a, 0xb6, 0x03, 0xb1, 0x47,
    0x4a, 0xfe, 0x17, 0x61,
])

class Test(unittest.TestCase):

    def test_init_raises(self):
        with self.assertRaises(ValueError) as context:
            test_random = gostcrypto.gostrandom.R132356510062017(32, TEST_SEED[:30], gostcrypto.gostrandom.SIZE_S_256)
        self.assertTrue('invalid seed value size' in str(context.exception))

    def test_random(self):
        test_random = gostcrypto.gostrandom.new(32, TEST_SEED, gostcrypto.gostrandom.SIZE_S_256)
        self.assertEqual(test_random.random(), TEST_RAND)
        test_random = gostcrypto.gostrandom.new(68, TEST_SEED, gostcrypto.gostrandom.SIZE_S_256)
        self.assertEqual(test_random.random(), TEST_RAND_LONG)

    def test_random_raises(self):
        test_random = gostcrypto.gostrandom.R132356510062017(32, TEST_SEED_ZERO, gostcrypto.gostrandom.SIZE_S_256)
        with self.assertRaises(ValueError) as context:
            test_random.random()
        self.assertTrue('the seed value is zero' in str(context.exception))
        test_random = gostcrypto.gostrandom.R132356510062017(32, TEST_SEED, gostcrypto.gostrandom.SIZE_S_256)
        test_random._limit = 0
        with self.assertRaises(ValueError) as context:
            test_random.random()
        self.assertTrue('exceeded the limit value of the counter' in str(context.exception))
        test_random = gostcrypto.gostrandom.R132356510062017(68, TEST_SEED, gostcrypto.gostrandom.SIZE_S_256)
        test_random._limit = 0
        with self.assertRaises(ValueError) as context:
            test_random.random()
        self.assertTrue('exceeded the limit value of the counter' in str(context.exception))

    def test_reset(self):
        test_random = gostcrypto.gostrandom.new(32, TEST_SEED_ZERO, gostcrypto.gostrandom.SIZE_S_256)
        test_random.reset(TEST_SEED)
        self.assertEqual(test_random.random(), TEST_RAND)

    def test_reset_raises(self):
        test_random = gostcrypto.gostrandom.new(32, TEST_SEED_ZERO, gostcrypto.gostrandom.SIZE_S_256)
        with self.assertRaises(ValueError) as context:
            test_random.reset(TEST_SEED[:30])
        self.assertTrue('invalid seed value size' in str(context.exception))

    def test_init_urandom(self):
        with mock.patch('os.urandom', os_urandom):
            test_random = gostcrypto.gostrandom.R132356510062017(32, None, gostcrypto.gostrandom.SIZE_S_256)
        self.assertEqual(test_random.random(), TEST_RAND)

    def test_reset_urandom(self):
        test_random = gostcrypto.gostrandom.new(32, TEST_SEED_ZERO, gostcrypto.gostrandom.SIZE_S_256)
        with mock.patch('os.urandom', os_urandom):
            test_random.reset()
        self.assertEqual(test_random.random(), TEST_RAND)



"""The module implementing the password-based key derivation function in accordance
   with R 50.1.111-2016.
"""

__title__ = 'gostpbkdf'
__author__ = 'Evgeny Drobotun'
__author_email__ = 'drobotun@xakep.ru'
__license__ = 'MIT'
__copyright__ = 'Copyright (C) 2020 Evgeny Drobotun'

from .r_50_1_111_2016 import *



"""The module that implements auxiliary functions for the operation of the
   GOSTcrypto module.

   Author: Evgeny Drobotun (c) 2020
   License: MIT

"""

__all__ = ['msb', 'add_xor', 'zero_fill',
           'bytearray_to_int', 'int_to_bytearray',
           'compare', 'compare_to_zero']

def msb(value):
    """Returns the value of the highest digit of the number 'value'.

       Args:
          :value: The number for which you want to determine the value of the high order.
    """
    return value[-1] & 0x80

def add_xor(op_a, op_b):
    """Byte-by-byte 'xor' operation for byte objects.

       Args:
          :op_a: The first operand.
          :op_b: The second operand.

       Return:
          Result of the byte-by-byte 'xor' operation.
    """
    op_a = bytearray(op_a)
    op_b = bytearray(op_b)
    result_len = min(len(op_a), len(op_b))
    result = bytearray(result_len)
    for i in range(result_len):
        result[i] = op_a[i] ^ op_b[i]
    return result

def zero_fill(size_object):
    """Zeroing byte objects.

       Args:
          :value: The size of byte object that you want to reset.
    """
    return b'/x00' * size_object

def bytearray_to_int(value):
    """Converting a 'bytearray' object to a long integer.

       Args:
          :value: The 'bytearray' object to convert.

       Return:
          Long integer value from 'bytearray' object.
    """
    return int.from_bytes(value, byteorder='big')

def int_to_bytearray(value, num_byte):
    """Converting a long integer to a 'bytearray' object.

       Args:
          :value: The long integer value to convert.
          :num_byte: Number of bytes in the 'bytearray'object.

       Return:
          'bytearray' object from long integer value.
    """
    return bytearray([(value &\
           (0xff << pos * 8)) >> pos * 8 for pos in range(num_byte - 1, -1, -1)])

def compare(op_a, op_b):
    """Comparing two byte arrays (function is performed in a constant time).

       Args:
          :op_a: First array to compare.
          :op_b: Second array to compare.

       Return:
          Сomparison result (boolean).
    """
    op_a = bytearray(op_a)
    op_b = bytearray(op_b)
    res_check = 0
    if len(op_a) != len(op_b):
        return False
    for i in enumerate(op_a):
        check = op_a[i[0]] ^ op_b[i[0]]
        res_check = res_check + check
    return not res_check

def compare_to_zero(value):
    """Comparing byte array with zero (function is performed in a constant time).

       Args:
          :value: Aarray to compare.

       Return:
          Сomparison result (boolean).
    """
    value = bytearray(value)
    res_check = 0
    for i in enumerate(value):
        check = value[i[0]] ^ 0x00
        res_check = res_check + check
    return not res_check



from setuptools import setup, find_packages
import gostcrypto

with open('README.rst', 'r', encoding='utf-8') as readme_file:
    readme = readme_file.read()
with open('HISTORY.rst', 'r', encoding='utf-8') as history_file:
    history = history_file.read()

setup(
    name='gostcrypto',
    version = gostcrypto.__version__,
    description = 'GOST cryptographic functions',
    long_description = readme + '\n\n' + history,
    author = gostcrypto.__author__,
    author_email = gostcrypto.__author_email__,
    url='https://github.com/drobotun/gostcrypto',
    zip_safe=False,
    license=gostcrypto.__license__,
    keywords='cryptography, hash function, encryption, digital signature, gost-r-34-12-2015, gost-r-34-10-2012, gost-r-34-13-2015, gost-r-34-11-2012',
    project_urls={
        'Documentation': 'https://gostcrypto.readthedocs.io/',
        'Source': 'https://github.com/drobotun/gostcrypto'
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.7',
    ],
    test_suite="tests",
    packages=find_packages()
    )



"""The package implements various cryptographic functions defined in the State standards
   of the Russian Federation. It includes the following modules:

      - gosthash: The module implements functions for calculating hash amounts in accordance
                  with GOST R 34.11-2012.

      - gostcipher: The module implements block encryption functions in accordance with
                    GOST R 34.12-2015 and their use modes in accordance with GOST R 34.13-2015.

      - gostsignature: The module implements the functions of forming and verifying an
                       electronic digital signature in accordance with GOST R 34.10-2012.

      - gostrandom: The module implements functions for generating pseudo-random sequences
                    in accordance with R 1323565.1.006-2017.

      - gosthmac: The module implements the functions of calculating the HMAC message
                  authentication code in accordance with R 50.1.113-2016.

      - gostpbkdf: The module implements the password-based key derivation function in
                   accordance with R 50.1.111-2016.
"""
__title__ = 'gostcrypto'
__version__ = '1.0.0'
__author__ = 'Evgeny Drobotun'
__author_email__ = 'drobotun@xakep.ru'
__license__ = 'MIT'
__copyright__ = 'Copyright (C) 2020 Evgeny Drobotun'

from .gosthash import *
from .gostcipher import *
from .gostsignature import *
from .gostrandom import *
from .gosthmac import *
from .gostpbkdf import *



"""The module implementing the password-based key derivation function in accordance
   with R 50.1.111-2016.

   Author: Evgeny Drobotun (c) 2020
   License: MIT

   Usage:

       import gostcrypto

       pbkdf_obj = new(<'password'>, <'salt'>)
       result = pbkdf_obj.derive(32)

"""
import os
from sys import exit as sys_exit

from gostcrypto.gosthmac import R5011132016
from gostcrypto.utils import int_to_bytearray
from gostcrypto.utils import zero_fill
from gostcrypto.utils import add_xor

__all__ = ['R5011112016', 'new']

_BLOCK_SIZE = 64

def new(password, salt=None, counter=1000):
    """Creates a new object for the password-based key derivation function and returns it.

       Args:
          :password: Password that is a character string in Unicode UTF-8 encoding.
          :salt: Random value. If this argument is not passed to the function, the 'os.urandom'
             function is used to generate this value with the length of the generated value of
             32 bytes.
          :counter: Number of iterations. The default value is 1000.

       Return:
          New object for the password-based key derivation function.

       Exception:
          ValueError('invalid password value') - in case of invalid password value.
    """
    try:
        return R5011112016(password, salt, counter)
    except ValueError as err:
        print(err)
        sys_exit()

class R5011112016:
    """Class that implementing the calculating the password-based key derivation function
       in accordance with R 50.1.111-2016.

       Methods:
          :derive(): returns a derived key as a byte object.
          :hexderive(): returns a derived key as a hexadecimal string.
          :clear(): clears the password value.

       Attributes:
          :salt: a byte object containing a random value (salt).
    """

    def __init__(self, password, salt, iterations):
        #Initialize the PBKDF object
        if not isinstance(password, (bytes, bytearray)):
            raise ValueError('ValueError: invalid password value')
        self._password = bytearray(password)
        password = zero_fill(len(password))
        if salt is None:
            self._salt = os.urandom(32)
        else:
            self._salt = bytearray(salt)
        self._iterations = iterations
        self._num_block = 0
        self._counter = 0
        self._hmac_obj = R5011132016('HMAC_GOSTR3411_2012_512', self._password)

    def __del__(self):
        #Delete the PBKDF object
        self.clear()

    def _u_first(self):
        #Primary calculation function U
        self._hmac_obj.reset()
        self._hmac_obj.update(self._salt + int_to_bytearray(self._counter, 4))
        return self._hmac_obj.digest()

    def _u_iter(self, u_prev):
        #Iterative calculation function U
        self._hmac_obj.reset()
        self._hmac_obj.update(u_prev)
        return self._hmac_obj.digest()

    def _f(self):
        #Iunction for calculating the PBKDF value once
        _t = self._u_first()
        internal = self._u_first()
        for _ in range(1, self._iterations):
            internal = self._u_iter(internal)
            _t = add_xor(_t, internal)
        return _t

    def _calculate_pbkdf(self, dk_len):
        #Function for final calculation of the PBKDF value
        result = b''
        self._num_block = dk_len // _BLOCK_SIZE
        if dk_len % _BLOCK_SIZE != 0:
            self._num_block = self._num_block + 1
        for i in range(1, self._num_block + 1):
            self._counter = i
            result = result + self._f()
        return result

    def derive(self, dk_len):
        """Returns a derived key as a byte object.

           Args:
              :dk_len: Required length of the output sequence (in bytes).

           Return:
              Derived key as a byte object with the length 'dk_len'.

           Exception:
              ValueError('invalid size of the derived key') - in case of invalid size of the
                derived key.
        """
        if dk_len > (2 ** 32 - 1) * 64:
            raise ValueError('ValueError: invalid size of the derived key')
        return self._calculate_pbkdf(dk_len)[:dk_len]

    def hexderive(self, dk_len):
        """Returns a derived key as a hexadecimal string.

           Args:
              :dk_len: Required length of the output sequence (in bytes).

           Return:
              Derived key as a hexadecimal string with the length 'dk_len'.

           Exception:
              ValueError('invalid size of the derived key') - in case of invalid size of the
                 derived key.
        """
        if dk_len > (2 ** 32 - 1) * 64:
            raise ValueError('ValueError: invalid size of the derived key')
        return self._calculate_pbkdf(dk_len)[:dk_len].hex()

    def clear(self):
        """Сlears the password value.
        """
        self._password = zero_fill(len(self._password))

    @property
    def salt(self):
        """The byte object containing a random value (salt).
        """
        return self._salt



"""The module that implements various block encryption modes (ECB, CBC, CFB, OFB, CTR and
   MAC according to GOST 34.13-2015.

   Author: Evgeny Drobotun (c) 2020
   License: MIT

   Usage:
    - encrypting a string:

        import gostcrypto

        CIPHER_KEY = bytearray([
            0xef, 0xcd, 0xab, 0x89, 0x67, 0x45, 0x23, 0x01,
            0x10, 0x32, 0x54, 0x76, 0x98, 0xba, 0xdc, 0xfe,
            0x77, 0x66, 0x55, 0x44, 0x33, 0x22, 0x11, 0x00,
            0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88,
        ])

        cipher = gostcrypto.gostcipher.new('kuznechik',
                                            CIPHER_KEY,
                                            gostcrypto.gostcipher.MODE_ECB)
        cipher_string = cipher.encrypt(b'<plain string>')

    - encrypting a file:

        import gostcrypto

        CIPHER_KEY = bytearray([
            0xef, 0xcd, 0xab, 0x89, 0x67, 0x45, 0x23, 0x01,
            0x10, 0x32, 0x54, 0x76, 0x98, 0xba, 0xdc, 0xfe,
            0x77, 0x66, 0x55, 0x44, 0x33, 0x22, 0x11, 0x00,
            0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88,
        ])

        CIPHER_IV = bytearray([
            0x12, 0x34, 0x56, 0x78, 0x90, 0xab, 0xce, 0xf0,
        ])

        cipher = gostcrypto.gostcipher.new('kuznechik',
                                            CIPHER_KEY,
                                            gostcrypto.gostcipher.MODE_CTR,
                                            init_vect=CIPHER_IV)

        #The 'buffer_size' must be a multiple of the block size
        buffer_size = 128
        file_in = open('<path to the plain text file>', 'rb')
        file_out = open('<path to the encrypted text file>', 'wb')
        buffer = file_in.read(buffer_size)
        while len(buffer) > 0:
            block = cipher.decrypt(buffer)
            file_out.write(block)
            buffer = file_in.read(buffer_size)

    - calculating MAC of the file:

        import gostcrypto

        CIPHER_KEY = bytearray([
            0xef, 0xcd, 0xab, 0x89, 0x67, 0x45, 0x23, 0x01,
            0x10, 0x32, 0x54, 0x76, 0x98, 0xba, 0xdc, 0xfe,
            0x77, 0x66, 0x55, 0x44, 0x33, 0x22, 0x11, 0x00,
            0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88,
        ])

        #The 'buffer_size' must be a multiple of the block size
        buffer_size = 128
        cipher = gostcrypto.gostcipher.new('kuznechik',
                                           CIPHER_KEY,
                                           gostcrypto.gostcipher.MODE_MAC,
                                           pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        file_in = open('<path to the file to calculate the MAC>', 'rb')
        buffer = file_in.read(buffer_size)
        while len(buffer) > 0:
            block = cipher.update(buffer)
            buffer = file_in.read(buffer_size)
        mac_result = cipher.digest(cipher.block_size)
"""

from sys import exit as sys_exit
from copy import deepcopy

from gostcrypto.utils import add_xor
from gostcrypto.utils import int_to_bytearray
from gostcrypto.utils import bytearray_to_int
from gostcrypto.utils import zero_fill
from gostcrypto.utils import msb

from .gost_34_12_2015 import GOST34122015Kuznechik
from .gost_34_12_2015 import GOST34122015Magma

__all__ = ['GOST34132015', 'new',
           'MODE_ECB', 'MODE_CBC',
           'MODE_CFB', 'MODE_OFB',
           'MODE_CTR', 'MODE_MAC',
           'PAD_MODE_1', 'PAD_MODE_2',
           'PAD_MODE_3']

MODE_ECB = 0x01 #Electronic Codebook mode
MODE_CBC = 0x02 #Cipher Block Chaining mode
MODE_CFB = 0x03 #Cipher Feedback mode
MODE_OFB = 0x05 #OutputFeedback mode
MODE_CTR = 0x06 #Counter mode
MODE_MAC = 0xff #Message Authentication Code algorithm

PAD_MODE_1 = 0x800000f0
PAD_MODE_2 = 0x800000f1
PAD_MODE_3 = 0x800000f2

_KEY_SIZE = 32

_B_64 = bytearray([
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1b,
])

_B_128 = bytearray([
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x87,
])

def new(algorithm, key, mode, **kwargs):
    """Creates a new ciphering object and returns it.

       Args:
          :algorithm: The string with the name of the ciphering algorithm
             of the GOST R 34.12-201 ('kuznechik' with block size 128 bit
             or 'magma' with block size 64 bit).
          :key: Byte object with 256-bit encryption key.
          :mode: Mode of operation of the block encryption algorithm (valid
             value: MODE_CBC, MODE_CFB, MODE_CTR, MODE_ECB, MODE_OFB or MODE_MAC).
       Keyword args:
          :init_vect: Byte object with initialization vector. Used in CTR, OFB,
             CBC and CFB modes. For CTR mode, the initialization vector length
             is equal to half the block size. For CBC, OFB and CFB modes, it
             is a multiple of the block size (the default value is 'None').
          :pad_mode: Padding mode for ECB, CBC, and MAC modes . For ECB and CBC modes,
             the acceptable values are PAD_MODE_1 and PAD_MODE_2. For MAC mode,
             the acceptable value is PAD_MODE_3 (the default value is PAD_MODE_1).

       Return:
          New ciphering object.

       Exception:
          ValueError('unsupported algorithm') - in case of invalid value 'algorithm'.
          ValueError('invalid key size') - in case of invalid key value.
          ValueError('invalid key value') - in case of invalid key value.
          ValueError('invalid IV value') - in case of invalid initialization vector value.
    """
    init_vect = kwargs.get('init_vect', None)
    pad_mode = kwargs.get('pad_mode', PAD_MODE_1)
    try:
        cipher_obj = GOST34132015(algorithm, key, mode,\
                                  init_vect=init_vect,\
                                  pad_mode=pad_mode)
        return cipher_obj
    except ValueError as err:
        print(err)
        sys_exit()

class GOST34132015:
    """Сlass that implements various block encryption modes in accordance with
       GOST 34.13-2015.

       Methods:
          :decrypt(): Decrypting a ciphertext.
          :encrypt(): Encrypting a plaintext.
          :update(): Update the MAC object with the bytes-like object.
          :digest(): Calculating the Message authentication code of the data passed to the
             'update()' method so far.
          :hexdigest(): Calculating the Message authentication code of the data passed to the
             'update()' method so far an return it of the hexadecimal.
          :clear(): Сlearing the values of iterative encryption keys.

       Attributes:
          :iv:  The initial value which will be used to start a cipher feedback mode.
          :counter: Counter blocks.
          :block_size: An integer value the internal block size of the cipher algorithm in bytes.
    """
    #pylint: disable=too-many-instance-attributes
    def __init__(self, algorithm, key, mode, **kwargs):
        #Initialize the ciphering object
        if not isinstance(key, (bytes, bytearray)):
            raise ValueError('ValueError: invalid key value')
        if len(key) != _KEY_SIZE:
            raise ValueError('ValueError: invalid key size')
        if not algorithm in ('magma', 'kuznechik'):
            key = zero_fill(len(key))
            raise ValueError('ValueError: unsupported algorithm')
        if algorithm == 'kuznechik':
            self._cipher_obj = GOST34122015Kuznechik(key)
        elif algorithm == 'magma':
            self._cipher_obj = GOST34122015Magma(key)
        self._mode = mode
        self._pad_mode = kwargs.get('pad_mode')
        self._init_vect = kwargs.get('init_vect')
        if mode in (MODE_CBC, MODE_CFB, MODE_OFB, MODE_CTR):
            if not isinstance(self._init_vect, (bytes, bytearray)):
                self.clear()
                raise ValueError('ValueError: invalid IV value')
            self._init_vect = bytearray(self._init_vect)
        if mode == MODE_CTR:
            self._counter = kwargs.get('init_vect') + b'\x00' * (self._cipher_obj.block_size // 2)
            self._counter = bytearray(self._counter)
        if mode == MODE_MAC:
            value_r = self._cipher_obj.encrypt(self._cipher_obj.block_size * b'\x00')
            self._key_1, self._key_2 = self._get_mac_key(value_r)
            self._buff = bytearray(self._cipher_obj.block_size)
            self._prev_mac = bytearray(self._cipher_obj.block_size)
            self._cur_mac = bytearray(self._cipher_obj.block_size)
        #pylint: enable=too-many-instance-attributes

    def __del__(self):
        #Delete the ciphering object
        self.clear()

    def _get_num_block(self, data):
        #Returns the number of blocks in the data
        return len(data) // self.block_size

    def _get_pad_size(self, data):
        #Returns the padding size
        if len(data) < self.block_size:
            result = self.block_size - len(data)
        elif len(data) % self.block_size == 0:
            result = 0
        else:
            result = self.block_size - len(data) % self.block_size
        return result

    def _pad_mode_1(self, data):
        #Setting of padding MODE_PAD_1
        return data + b'\x00' * self._get_pad_size(data)

    def _pad_mode_2(self, data):
        #Setting of padding MODE_PAD_2
        return data + b'\x80' + b'\x00' * (self.block_size + self._get_pad_size(data) - 1)

    def _pad_mode_3(self, data):
        #Setting of padding MODE_PAD_3
        if self._get_pad_size(data) == 0:
            result = data
        else:
            result = data + b'\x80' + b'\x00' * (self._get_pad_size(data) - 1)
        return result

    def _set_padding(self, data):
        #Selecting and setting padding
        if self._pad_mode == PAD_MODE_1:
            result = self._pad_mode_1(data)
        elif self._pad_mode == PAD_MODE_2:
            result = self._pad_mode_2(data)
        else:
            result = self._pad_mode_3(data)
        return result

    def _inc_ctr(self, ctr):
        #Increasing the counter value in CTR mode
        internal = 0
        bit = bytearray(self.block_size)
        bit[self.block_size - 1] = 0x01
        for i in range(self.block_size):
            internal = ctr[i] + bit[i] + (internal << 8)
            ctr[i] = internal & 0xff
        return ctr

    def _get_mac_key(self, value_r):
        #Generating final keys for MAC mode
        if self.block_size == 16:
            value_b = _B_128
        else:
            value_b = _B_64
        if msb(value_r) == 0:
            int_value = bytearray_to_int(value_r) << 1
            key_1 = int_to_bytearray(int_value, self.block_size)
        else:
            int_value = bytearray_to_int(value_r) << 1
            key_1 = add_xor(int_to_bytearray(int_value, self.block_size), value_b)
        if msb(key_1) == 0:
            int_value = bytearray_to_int(key_1) << 1
            key_2 = int_to_bytearray(int_value, self.block_size)
        else:
            int_value = bytearray_to_int(key_1) << 1
            key_2 = add_xor(int_to_bytearray(int_value, self.block_size), value_b)
        return [key_1, key_2]

    def _ecb_decrypt(self, data):
        #Decryption mode ECB
        if not self._pad_mode in (PAD_MODE_1, PAD_MODE_2):
            self.clear()
            raise ValueError('ValueError: unsupported padding mode')
        result = bytearray()
        for i in range(self._get_num_block(data)):
            result = (result + self._cipher_obj.decrypt(data[self.block_size *\
                      i:self.block_size + (self.block_size * i):]))
        return result

    def _cbc_decrypt(self, data):
        #Decryption mode CBC
        if not self._pad_mode in (PAD_MODE_1, PAD_MODE_2):
            self.clear()
            raise ValueError('ValueError: unsupported padding mode')
        if not self._init_vect or len(self._init_vect) < self.block_size\
                               or len(self._init_vect) % self.block_size != 0:
            self.clear()
            raise ValueError('ValueError: invalid initialization vector size')
        result = bytearray()
        for i in range(self._get_num_block(data)):
            cipher_blk = add_xor(\
                         self._init_vect[0:self.block_size], self._cipher_obj.decrypt(\
                         data[self.block_size * i:self.block_size + (self.block_size * i)]))
            result = result + cipher_blk
            self._init_vect[0:len(self._init_vect) - self.block_size] =\
                           self._init_vect[self.block_size:len(self._init_vect)]
            self._init_vect[len(self._init_vect) - self.block_size:len(self._init_vect)] =\
                           data[self.block_size * i:self.block_size + (self.block_size * i)]
        return result

    def _cfb_decrypt(self, data):
        #Decryption mode CFB
        if not self._init_vect or len(self._init_vect) < self.block_size\
                               or len(self._init_vect) % self.block_size != 0:
            self.clear()
            raise ValueError('ValueError: invalid initialization vector size')
        gamma = bytearray()
        result = bytearray()
        for i in range(self._get_num_block(data)):
            gamma = self._cipher_obj.encrypt(self._init_vect[0:self.block_size])
            cipher_blk = data[self.block_size * i:self.block_size + self.block_size * i]
            result = result + add_xor(gamma, cipher_blk)
            self._init_vect[0:len(self._init_vect) - self.block_size] =\
                           self._init_vect[self.block_size:len(self._init_vect)]
            self._init_vect[len(self._init_vect) - self.block_size:len(self._init_vect)] =\
                           cipher_blk
        if len(data) % self.block_size != 0:
            gamma = self._cipher_obj.encrypt(self._init_vect[0:self.block_size])
            cipher_blk = add_xor(gamma, data[self.block_size * self._get_num_block(data)::])
            result = result + cipher_blk
        return result

    def _ofb_decrypt(self, data):
        #Decryption mode OFB
        if not self._init_vect or len(self._init_vect) < self.block_size\
                               or len(self._init_vect) % self.block_size != 0:
            self.clear()
            raise ValueError('ValueError: invalid initialization vector size')
        gamma = bytearray()
        result = bytearray()
        for i in range(self._get_num_block(data)):
            gamma = self._cipher_obj.encrypt(self._init_vect[0:self.block_size])
            cipher_blk = add_xor(\
                         gamma, data[self.block_size * i:self.block_size + self.block_size * i])
            result = result + cipher_blk
            self._init_vect[0:len(self._init_vect) - self.block_size] =\
                           self._init_vect[self.block_size:len(self._init_vect)]
            self._init_vect[len(self._init_vect) - self.block_size:len(self._init_vect)] =\
                           gamma[0:self.block_size]
        if len(data) % self.block_size != 0:
            gamma = self._cipher_obj.encrypt(self._init_vect[0:self.block_size])
            cipher_blk = add_xor(gamma, data[self.block_size * self._get_num_block(data)::])
            result = result + cipher_blk
        return result

    def _ctr_decrypt(self, data):
        #Decryption mode CTR
        if not self._init_vect or len(self._init_vect) != self.block_size // 2:
            self.clear()
            raise ValueError('ValueError: invalid initialization vector size')
        gamma = bytearray()
        result = bytearray()
        for i in range(self._get_num_block(data)):
            gamma = self._cipher_obj.encrypt(self._counter)
            self._counter = self._inc_ctr(self._counter)
            result = result + add_xor(\
                     data[self.block_size * i:self.block_size + (self.block_size * i)], gamma)
        if len(data) % self.block_size != 0:
            gamma = self._cipher_obj.encrypt(self._counter)
            self._counter = self._inc_ctr(self._counter)
            result = result +\
                     add_xor(data[self.block_size * self._get_num_block(data)::], gamma)
        return result

    def _ecb_encrypt(self, data):
        #Encryption mode ECB
        if not self._pad_mode in (PAD_MODE_1, PAD_MODE_2):
            self.clear()
            raise ValueError('ValueError: unsupported padding mode')
        data = self._set_padding(data)
        result = bytearray()
        for i in range(self._get_num_block(data)):
            result = result + self._cipher_obj.encrypt(data[self.block_size *\
                     i:self.block_size + (self.block_size * i):1])
        return result

    def _cbc_encrypt(self, data):
        #Encryption mode CBC
        if not self._pad_mode in (PAD_MODE_1, PAD_MODE_2):
            self.clear()
            raise ValueError('ValueError: unsupported padding mode')
        if not self._init_vect or len(self._init_vect) < self.block_size\
                               or len(self._init_vect) % self.block_size != 0:
            self.clear()
            raise ValueError('ValueError: invalid initialization vector size')
        data = self._set_padding(data)
        result = bytearray()
        for i in range(self._get_num_block(data)):
            cipher_blk = self._cipher_obj.encrypt(add_xor(self._init_vect[0:self.block_size:],\
                         data[self.block_size * i:self.block_size + (self.block_size * i)]))
            result = result + cipher_blk
            self._init_vect[0:len(self._init_vect) - self.block_size] =\
                           self._init_vect[self.block_size:len(self._init_vect)]
            self._init_vect[len(self._init_vect) - self.block_size:len(self._init_vect)] =\
                           cipher_blk[0:self.block_size]
        return result

    def _cfb_encrypt(self, data):
        #Encryption mode CFB
        if not self._init_vect or len(self._init_vect) < self.block_size\
                               or len(self._init_vect) % self.block_size != 0:
            self.clear()
            raise ValueError('ValueError: invalid initialization vector size')
        gamma = bytearray()
        result = bytearray()
        for i in range(self._get_num_block(data)):
            gamma = self._cipher_obj.encrypt(self._init_vect[0:self.block_size])
            cipher_blk = add_xor(\
                         gamma, data[self.block_size * i:self.block_size + (self.block_size * i)])
            result = result + cipher_blk
            self._init_vect[0:len(self._init_vect) - self.block_size] =\
                           self._init_vect[self.block_size:len(self._init_vect)]
            self._init_vect[len(self._init_vect) - self.block_size:len(self._init_vect)] =\
                           cipher_blk[0:self.block_size]
        if len(data) % self.block_size != 0:
            gamma = self._cipher_obj.encrypt(self._init_vect[0:self.block_size])
            cipher_blk = add_xor(gamma, data[self.block_size * self._get_num_block(data)::])
            result = result + cipher_blk
        return result

    def _ofb_encrypt(self, data):
        #Encryption mode OFB
        if not self._init_vect or len(self._init_vect) < self.block_size\
                               or len(self._init_vect) % self.block_size != 0:
            self.clear()
            raise ValueError('ValueError: invalid initialization vector size')
        gamma = bytearray()
        result = bytearray()
        for i in range(self._get_num_block(data)):
            gamma = self._cipher_obj.encrypt(self._init_vect[0:self.block_size])
            cipher_blk = add_xor(\
                         gamma, data[self.block_size * i:self.block_size + (self.block_size * i)])
            result = result + cipher_blk
            self._init_vect[0:len(self._init_vect) - self.block_size] =\
                           self._init_vect[self.block_size:len(self._init_vect)]
            self._init_vect[len(self._init_vect) - self.block_size:len(self._init_vect)] =\
                           gamma[0:self.block_size]
        if len(data) % self.block_size != 0:
            gamma = self._cipher_obj.encrypt(self._init_vect[0:self.block_size])
            cipher_blk = add_xor(gamma, data[self.block_size * self._get_num_block(data)::])
            result = result + cipher_blk
        return result

    def _ctr_encrypt(self, data):
        #Encryption mode CTR
        if not self._init_vect or len(self._init_vect) != (self.block_size // 2):
            self.clear()
            raise ValueError('ValueError: 66invalid initialization vector size')
        gamma = bytearray()
        result = bytearray()
        for i in range(self._get_num_block(data)):
            gamma = self._cipher_obj.encrypt(self._counter)
            self._counter = self._inc_ctr(self._counter)
            result = result + add_xor(\
                     data[self.block_size * i:self.block_size + (self.block_size * i)], gamma)
        if len(data) % self.block_size != 0:
            gamma = self._cipher_obj.encrypt(self._counter)
            self._counter = self._inc_ctr(self._counter)
            result = result + add_xor(\
                     data[self.block_size * self._get_num_block(data)::], gamma)
        return result

    def decrypt(self, data):
        """Decrypting a ciphertext.

           Args:
             :data: Ciphertext data to be decrypted.

           Return:
              Plaintext data.

           Exception:
              ValueError('unsupported cipher mode') - in case of the unsupported cipher mode.
              ValueError('unsupported padding mode') - in case of the unsupported padding mode.
              ValueError('invalid initialization vector size') - in case of the
                 invalid initialization vector size.
        """
        if not self._mode in (MODE_ECB, MODE_CBC, MODE_CFB, MODE_OFB, MODE_CTR):
            self.clear()
            raise ValueError('ValueError: unsupported cipher mode')
        if self._mode == MODE_ECB:
            result = self._ecb_decrypt(data)
        elif self._mode == MODE_CBC:
            result = self._cbc_decrypt(data)
        elif self._mode == MODE_CFB:
            result = self._cfb_decrypt(data)
        elif self._mode == MODE_OFB:
            result = self._ofb_decrypt(data)
        else:
            result = self._ctr_decrypt(data)
        return result

    def encrypt(self, data):
        """Encrypting a plaintext.

           Args:
             :data: Plaintext data to be encrypted.

           Return:
              Ciphertext data.

           Exception:
              ValueError('unsupported cipher mode') - in case of the unsupported cipher mode.
              ValueError('unsupported padding mode') - in case of the unsupported padding mode.
              ValueError('invalid initialization vector size') - in case of the
                 invalid initialization vector size.
        """
        if not self._mode in (MODE_ECB, MODE_CBC, MODE_CFB, MODE_OFB, MODE_CTR):
            self.clear()
            raise ValueError('ValueError: unsupported cipher mode')
        if self._mode == MODE_ECB:
            result = self._ecb_encrypt(data)
        elif self._mode == MODE_CBC:
            result = self._cbc_encrypt(data)
        elif self._mode == MODE_CFB:
            result = self._cfb_encrypt(data)
        elif self._mode == MODE_OFB:
            result = self._ofb_encrypt(data)
        else:
            result = self._ctr_encrypt(data)
        return result

    def update(self, data):
        """Update the MAC object with the bytes-like object.

           Args:
              :data: The string from which to get the MAC. Repeated calls are equivalent
                to a single call with the concatenation of all the arguments: m.update(a);
                m.update(b) is equivalent to m.update(a+b).
 
           Exception:
              ValueError('unsupported cipher mode') - in case of the unsupported cipher mode.
              ValueError('unsupported padding mode') - in case of the unsupported padding mode.
        """
        if self._mode != MODE_MAC:
            self.clear()
            raise ValueError('ValueError: unsupported cipher mode')
        if self._pad_mode != PAD_MODE_3:
            self.clear()
            raise ValueError('ValueError: unsupported padding mode')
        data = self._set_padding(data)
        block = bytearray()
        prev_block = self._cur_mac
        for i in range(0, self._get_num_block(data)-1):
            block = self._cipher_obj.encrypt(add_xor(\
                    prev_block, data[self.block_size * i:self.block_size + (self.block_size * i)]))
            prev_block = block
        block = self._cipher_obj.encrypt(add_xor(\
                prev_block, data[len(data) - self.block_size:len(data)]))
        self._cur_mac = block
        self._prev_mac = prev_block
        self._buff = data[self.block_size * (self._get_num_block(data) - 1):]

    def mac_final(self):
        """Return the final value of the MAC.
        """
        if self._mode != MODE_MAC:
            self.clear()
            raise ValueError('ValueError: unsupported cipher mode')
        if self._pad_mode != PAD_MODE_3:
            self.clear()
            raise ValueError('ValueError: unsupported padding mode')
        if self._get_pad_size(self._buff) == 0:
            final_key = self._key_1
        else:
            final_key = self._key_2
        self._buff = self._set_padding(self._buff)
        result = bytearray()
        result = self._cipher_obj.encrypt(add_xor(add_xor(\
                 self._prev_mac, self._buff), final_key))
        return result

    def digest(self, mac_size):
        """Calculating the Message authentication code (MAC).

           Args:
             :mac_size: Message authentication code size (in bytes).

           Return:
              Message authentication code value.

           Exception:
              ValueError('unsupported cipher mode') - in case of the unsupported cipher mode.
              ValueError('unsupported padding mode') - in case of the unsupported padding mode.
              ValueError('invalid message authentication code size') - in case of the invalid
                 message authentication code size.
        """
        temp = deepcopy(self)
        if mac_size > temp.block_size:
            temp.clear()
            raise ValueError('ValueError: invalid message authentication code size')
        return temp.mac_final()[0:mac_size:]

    def hexdigest(self, mac_size):
        """Calculating the Message authentication code of the data passed to the
             'update()' method so far an return it of the hexadecimal.

           Args:
             :mac_size: Message authentication code size (in bytes).

           Return:
              Message authentication code value in hexadecimal.

           Exception:
              ValueError('unsupported cipher mode') - in case of the unsupported cipher mode.
              ValueError('unsupported padding mode') - in case of the unsupported padding mode.
              ValueError('invalid message authentication code size') - in case of the invalid
                 message authentication code size
        """
        return self.digest(mac_size).hex()

    #pylint: disable=invalid-name
    @property
    def iv(self):
        """Contains the initial value which will be used to start a cipher feedback
           mode (CBC, CFB or OFB mode only).

           Exception:
              ValueError('invalid IV value') - in case of the invalid IV value.
        """
        if self._mode in (MODE_ECB, MODE_CTR):
            self.clear()
            raise ValueError('ValueError: invalid IV value')
        return self._init_vect[len(self._init_vect) - self.block_size::]
    #pylint: enable=invalid-name

    @property
    def counter(self):
        """Contains counter blocks (CTR mode only).

           Exception:
              ValueError('invalid counter value') - in case of the invalid counter value.
        """
        if self._mode != MODE_CTR:
            self.clear()
            raise ValueError('ValueError: invalid counter value')
        return self._counter

    @property
    def block_size(self):
        """An integer value the internal block size of the cipher algorithm in bytes.
           For the 'Kuznechik' algorithm this value is 16 and the 'Magma' algorithm,
           this value is 8.
        """
        return self._cipher_obj.block_size

    def clear(self):
        """Сlearing the values of iterative encryption keys.
        """
        if hasattr(self, '_cipher_obj'):
            self._cipher_obj.clear()



"""The module that implements processes for creating and verifying an electronic digital
   signature according to GOST 34.10-2012.

   Author: Evgeny Drobotun (c) 2020
   License: MIT

   Usage:
    - signing:

       import gostcrypto

       private_key = bytearray.fromhex(
                     '7a929ade789bb9be10ed359dd39a72c11b60961f49397eee1d19ce9891ec3b28')
       digest = bytearray.fromhex(
                '2dfbc1b372d89a1188c09c52e0eec61fce52032ab1022e8e67ece6672b043ee5')

       sign_obj = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_256,
                  gostcrypto.gostsignature.CURVES_R_1323565_1_024_2019
                  ['id-tc26-gost-3410-2012-256-paramSetB'])
       signature = sign_obj.sign(private_key, digest)

    - verify:

       import gostcrypto

       public_key = bytearray.fromhex(
                    '7f2b49e270db6d90d8595bec458b50c58585ba1d4e9b788f6689dbd8e56fd80b\
                     26f1b489d6701dd185c8413a977b3cbbaf64d1c593d26627dffb101a87ff77da')
       digest = bytearray.fromhex(
                '2dfbc1b372d89a1188c09c52e0eec61fce52032ab1022e8e67ece6672b043ee5')
       signature = bytearray.fromhex(
                   '41aa28d2f1ab148280cd9ed56feda41974053554a42767b83ad043fd39dc0493\
                    01456c64ba4642a1653c235a98a60249bcd6d3f746b631df928014f6c5bf9c40')

       sign_obj = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_256,
                  gostcrypto.gostsignature.CURVES_R_1323565_1_024_2019
                  ['id-tc26-gost-3410-2012-256-paramSetB'])
       if sign_obj.verify(public_key, digest, signature):
           print('Signature is correct')
       else:
           print('Signature is not correct')

    - generating a public key:

       import gostcrypto

       private_key = bytearray.fromhex(
                     '7a929ade789bb9be10ed359dd39a72c11b60961f49397eee1d19ce9891ec3b28')

       sign_obj = gostcrypto.gostsignature.new(gostcrypto.gostsignature.MODE_256,
                  gostcrypto.gostsignature.CURVES_R_1323565_1_024_2019
                  ['id-tc26-gost-3410-2012-256-paramSetB'])
       public_key = sign_obj.public_key_generate(private_key)

"""
import os
from sys import exit as sys_exit

from gostcrypto.utils import zero_fill
from gostcrypto.utils import bytearray_to_int
from gostcrypto.utils import int_to_bytearray
from gostcrypto.utils import compare
from gostcrypto.utils import compare_to_zero

MODE_256 = 0x01
MODE_512 = 0x02

__all__ = ['GOST34102012', 'new',
           'MODE_256', 'MODE_512',
           'CURVES_R_1323565_1_024_2019']

#Parameters of elliptic curves in accordance with R 1323565.1.024-2019
CURVES_R_1323565_1_024_2019 = {
    'id-tc26-gost-3410-2012-256-paramSetB': dict(
        p=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
            0x97
        ])),
        a=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
            0x94
        ])),
        b=0xa6,
        m=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0x6c, 0x61, 0x10, 0x70, 0x99, 0x5a, 0xd1,
            0x00, 0x45, 0x84, 0x1b, 0x09, 0xb7, 0x61, 0xb8,
            0x93
        ])),
        q=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0x6c, 0x61, 0x10, 0x70, 0x99, 0x5a, 0xd1,
            0x00, 0x45, 0x84, 0x1b, 0x09, 0xb7, 0x61, 0xb8,
            0x93
        ])),
        x=0x01,
        y=bytearray_to_int(bytearray([
            0x00, 0x8d, 0x91, 0xe4, 0x71, 0xe0, 0x98, 0x9c,
            0xda, 0x27, 0xdf, 0x50, 0x5a, 0x45, 0x3f, 0x2b,
            0x76, 0x35, 0x29, 0x4f, 0x2d, 0xdf, 0x23, 0xe3,
            0xb1, 0x22, 0xac, 0xc9, 0x9c, 0x9e, 0x9f, 0x1e,
            0x14
        ]))
    ),
    'id-tc26-gost-3410-2012-256-paramSetC': dict(
        p=bytearray_to_int(bytearray([
            0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0c,
            0x99
        ])),
        a=bytearray_to_int(bytearray([
            0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0c,
            0x96
        ])),
        b=bytearray_to_int(bytearray([
            0x3e, 0x1a, 0xf4, 0x19, 0xa2, 0x69, 0xa5, 0xf8,
            0x66, 0xa7, 0xd3, 0xc2, 0x5c, 0x3d, 0xf8, 0x0a,
            0xe9, 0x79, 0x25, 0x93, 0x73, 0xff, 0x2b, 0x18,
            0x2f, 0x49, 0xd4, 0xce, 0x7e, 0x1b, 0xbc, 0x8b
        ])),
        m=bytearray_to_int(bytearray([
            0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x01, 0x5f, 0x70, 0x0c, 0xff, 0xf1, 0xa6, 0x24,
            0xe5, 0xe4, 0x97, 0x16, 0x1b, 0xcc, 0x8a, 0x19,
            0x8f
        ])),
        q=bytearray_to_int(bytearray([
            0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x01, 0x5f, 0x70, 0x0c, 0xff, 0xf1, 0xa6, 0x24,
            0xe5, 0xe4, 0x97, 0x16, 0x1b, 0xcc, 0x8a, 0x19,
            0x8f
        ])),
        x=0x01,
        y=bytearray_to_int(bytearray([
            0x3f, 0xa8, 0x12, 0x43, 0x59, 0xf9, 0x66, 0x80,
            0xb8, 0x3d, 0x1c, 0x3e, 0xb2, 0xc0, 0x70, 0xe5,
            0xc5, 0x45, 0xc9, 0x85, 0x8d, 0x03, 0xec, 0xfb,
            0x74, 0x4b, 0xf8, 0xd7, 0x17, 0x71, 0x7e, 0xfc
        ]))
    ),
    'id-tc26-gost-3410-2012-256-paramSetD': dict(
        p=bytearray_to_int(bytearray([
            0x00, 0x9b, 0x9f, 0x60, 0x5f, 0x5a, 0x85, 0x81,
            0x07, 0xab, 0x1e, 0xc8, 0x5e, 0x6b, 0x41, 0xc8,
            0xaa, 0xcf, 0x84, 0x6e, 0x86, 0x78, 0x90, 0x51,
            0xd3, 0x79, 0x98, 0xf7, 0xb9, 0x02, 0x2d, 0x75,
            0x9b
        ])),
        a=bytearray_to_int(bytearray([
            0x00, 0x9b, 0x9f, 0x60, 0x5f, 0x5a, 0x85, 0x81,
            0x07, 0xab, 0x1e, 0xc8, 0x5e, 0x6b, 0x41, 0xc8,
            0xaa, 0xcf, 0x84, 0x6e, 0x86, 0x78, 0x90, 0x51,
            0xd3, 0x79, 0x98, 0xf7, 0xb9, 0x02, 0x2d, 0x75,
            0x98
        ])),
        b=bytearray_to_int(bytearray([
            0x80, 0x5a
        ])),
        m=bytearray_to_int(bytearray([
            0x00, 0x9b, 0x9f, 0x60, 0x5f, 0x5a, 0x85, 0x81,
            0x07, 0xab, 0x1e, 0xc8, 0x5e, 0x6b, 0x41, 0xc8,
            0xaa, 0x58, 0x2c, 0xa3, 0x51, 0x1e, 0xdd, 0xfb,
            0x74, 0xf0, 0x2f, 0x3a, 0x65, 0x98, 0x98, 0x0b,
            0xb9
        ])),
        q=bytearray_to_int(bytearray([
            0x00, 0x9b, 0x9f, 0x60, 0x5f, 0x5a, 0x85, 0x81,
            0x07, 0xab, 0x1e, 0xc8, 0x5e, 0x6b, 0x41, 0xc8,
            0xaa, 0x58, 0x2c, 0xa3, 0x51, 0x1e, 0xdd, 0xfb,
            0x74, 0xf0, 0x2f, 0x3a, 0x65, 0x98, 0x98, 0x0b,
            0xb9
        ])),
        x=0x00,
        y=bytearray_to_int(bytearray([
            0x41, 0xec, 0xe5, 0x57, 0x43, 0x71, 0x1a, 0x8c,
            0x3c, 0xbf, 0x37, 0x83, 0xcd, 0x08, 0xc0, 0xee,
            0x4d, 0x4d, 0xc4, 0x40, 0xd4, 0x64, 0x1a, 0x8f,
            0x36, 0x6e, 0x55, 0x0d, 0xfd, 0xb3, 0xbb, 0x67
        ]))
    ),
    'id-tc26-gost-3410-12-512-paramSetA': dict(
        p=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
            0xc7
        ])),
        a=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
            0xc4
        ])),
        b=bytearray_to_int(bytearray([
            0x00, 0xe8, 0xc2, 0x50, 0x5d, 0xed, 0xfc, 0x86,
            0xdd, 0xc1, 0xbd, 0x0b, 0x2b, 0x66, 0x67, 0xf1,
            0xda, 0x34, 0xb8, 0x25, 0x74, 0x76, 0x1c, 0xb0,
            0xe8, 0x79, 0xbd, 0x08, 0x1c, 0xfd, 0x0b, 0x62,
            0x65, 0xee, 0x3c, 0xb0, 0x90, 0xf3, 0x0d, 0x27,
            0x61, 0x4c, 0xb4, 0x57, 0x40, 0x10, 0xda, 0x90,
            0xdd, 0x86, 0x2e, 0xf9, 0xd4, 0xeb, 0xee, 0x47,
            0x61, 0x50, 0x31, 0x90, 0x78, 0x5a, 0x71, 0xc7,
            0x60
        ])),
        m=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0x27, 0xe6, 0x95, 0x32, 0xf4, 0x8d, 0x89,
            0x11, 0x6f, 0xf2, 0x2b, 0x8d, 0x4e, 0x05, 0x60,
            0x60, 0x9b, 0x4b, 0x38, 0xab, 0xfa, 0xd2, 0xb8,
            0x5d, 0xca, 0xcd, 0xb1, 0x41, 0x1f, 0x10, 0xb2,
            0x75
        ])),
        q=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0x27, 0xe6, 0x95, 0x32, 0xf4, 0x8d, 0x89,
            0x11, 0x6f, 0xf2, 0x2b, 0x8d, 0x4e, 0x05, 0x60,
            0x60, 0x9b, 0x4b, 0x38, 0xab, 0xfa, 0xd2, 0xb8,
            0x5d, 0xca, 0xcd, 0xb1, 0x41, 0x1f, 0x10, 0xb2,
            0x75
        ])),
        x=0x03,
        y=bytearray_to_int(bytearray([
            0x75, 0x03, 0xcf, 0xe8, 0x7a, 0x83, 0x6a, 0xe3,
            0xa6, 0x1b, 0x88, 0x16, 0xe2, 0x54, 0x50, 0xe6,
            0xce, 0x5e, 0x1c, 0x93, 0xac, 0xf1, 0xab, 0xc1,
            0x77, 0x80, 0x64, 0xfd, 0xcb, 0xef, 0xa9, 0x21,
            0xdf, 0x16, 0x26, 0xbe, 0x4f, 0xd0, 0x36, 0xe9,
            0x3d, 0x75, 0xe6, 0xa5, 0x0e, 0x3a, 0x41, 0xe9,
            0x80, 0x28, 0xfe, 0x5f, 0xc2, 0x35, 0xf5, 0xb8,
            0x89, 0xa5, 0x89, 0xcb, 0x52, 0x15, 0xf2, 0xa4
        ]))
    ),
    'id-tc26-gost-3410-12-512-paramSetB': dict(
        p=bytearray_to_int(bytearray([
            0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x6f
        ])),
        a=bytearray_to_int(bytearray([
            0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x6c
        ])),
        b=bytearray_to_int(bytearray([
            0x68, 0x7d, 0x1b, 0x45, 0x9d, 0xc8, 0x41, 0x45,
            0x7e, 0x3e, 0x06, 0xcf, 0x6f, 0x5e, 0x25, 0x17,
            0xb9, 0x7c, 0x7d, 0x61, 0x4a, 0xf1, 0x38, 0xbc,
            0xbf, 0x85, 0xdc, 0x80, 0x6c, 0x4b, 0x28, 0x9f,
            0x3e, 0x96, 0x5d, 0x2d, 0xb1, 0x41, 0x6d, 0x21,
            0x7f, 0x8b, 0x27, 0x6f, 0xad, 0x1a, 0xb6, 0x9c,
            0x50, 0xf7, 0x8b, 0xee, 0x1f, 0xa3, 0x10, 0x6e,
            0xfb, 0x8c, 0xcb, 0xc7, 0xc5, 0x14, 0x01, 0x16
        ])),
        m=bytearray_to_int(bytearray([
            0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x01, 0x49, 0xa1, 0xec, 0x14, 0x25, 0x65, 0xa5,
            0x45, 0xac, 0xfd, 0xb7, 0x7b, 0xd9, 0xd4, 0x0c,
            0xfa, 0x8b, 0x99, 0x67, 0x12, 0x10, 0x1b, 0xea,
            0x0e, 0xc6, 0x34, 0x6c, 0x54, 0x37, 0x4f, 0x25,
            0xbd
        ])),
        q=bytearray_to_int(bytearray([
            0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x01, 0x49, 0xa1, 0xec, 0x14, 0x25, 0x65, 0xa5,
            0x45, 0xac, 0xfd, 0xb7, 0x7b, 0xd9, 0xd4, 0x0c,
            0xfa, 0x8b, 0x99, 0x67, 0x12, 0x10, 0x1b, 0xea,
            0x0e, 0xc6, 0x34, 0x6c, 0x54, 0x37, 0x4f, 0x25,
            0xbd
        ])),
        x=0x02,
        y=bytearray_to_int(bytearray([
            0x1a, 0x8f, 0x7e, 0xda, 0x38, 0x9b, 0x09, 0x4c,
            0x2c, 0x07, 0x1e, 0x36, 0x47, 0xa8, 0x94, 0x0f,
            0x3c, 0x12, 0x3b, 0x69, 0x75, 0x78, 0xc2, 0x13,
            0xbe, 0x6d, 0xd9, 0xe6, 0xc8, 0xec, 0x73, 0x35,
            0xdc, 0xb2, 0x28, 0xfd, 0x1e, 0xdf, 0x4a, 0x39,
            0x15, 0x2c, 0xbc, 0xaa, 0xf8, 0xc0, 0x39, 0x88,
            0x28, 0x04, 0x10, 0x55, 0xf9, 0x4c, 0xee, 0xec,
            0x7e, 0x21, 0x34, 0x07, 0x80, 0xfe, 0x41, 0xbd
        ]))
    ),
    'id-tc26-gost-3410-2012-256-paramSetA': dict(
        p=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
            0x97
        ])),
        a=bytearray_to_int(bytearray([
            0x00, 0xc2, 0x17, 0x3f, 0x15, 0x13, 0x98, 0x16,
            0x73, 0xaf, 0x48, 0x92, 0xc2, 0x30, 0x35, 0xa2,
            0x7c, 0xe2, 0x5e, 0x20, 0x13, 0xbf, 0x95, 0xaa,
            0x33, 0xb2, 0x2c, 0x65, 0x6f, 0x27, 0x7e, 0x73,
            0x35
        ])),
        b=bytearray_to_int(bytearray([
            0x29, 0x5f, 0x9b, 0xae, 0x74, 0x28, 0xed, 0x9c,
            0xcc, 0x20, 0xe7, 0xc3, 0x59, 0xa9, 0xd4, 0x1a,
            0x22, 0xfc, 0xcd, 0x91, 0x08, 0xe1, 0x7b, 0xf7,
            0xba, 0x93, 0x37, 0xa6, 0xf8, 0xae, 0x95, 0x13
        ])),
        e=0x01,
        d=bytearray_to_int(bytearray([
            0x06, 0x05, 0xf6, 0xb7, 0xc1, 0x83, 0xfa, 0x81,
            0x57, 0x8b, 0xc3, 0x9c, 0xfa, 0xd5, 0x18, 0x13,
            0x2b, 0x9d, 0xf6, 0x28, 0x97, 0x00, 0x9a, 0xf7,
            0xe5, 0x22, 0xc3, 0x2d, 0x6d, 0xc7, 0xbf, 0xfb
        ])),
        m=bytearray_to_int(bytearray([
            0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x3f, 0x63, 0x37, 0x7f, 0x21, 0xed, 0x98,
            0xd7, 0x04, 0x56, 0xbd, 0x55, 0xb0, 0xd8, 0x31,
            0x9c
        ])),
        q=bytearray_to_int(bytearray([
            0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x0f, 0xd8, 0xcd, 0xdf, 0xc8, 0x7b, 0x66, 0x35,
            0xc1, 0x15, 0xaf, 0x55, 0x6c, 0x36, 0x0c, 0x67
        ])),
        x=bytearray_to_int(bytearray([
            0x00, 0x91, 0xe3, 0x84, 0x43, 0xa5, 0xe8, 0x2c,
            0x0d, 0x88, 0x09, 0x23, 0x42, 0x57, 0x12, 0xb2,
            0xbb, 0x65, 0x8b, 0x91, 0x96, 0x93, 0x2e, 0x02,
            0xc7, 0x8b, 0x25, 0x82, 0xfe, 0x74, 0x2d, 0xaa,
            0x28
        ])),
        y=bytearray_to_int(bytearray([
            0x32, 0x87, 0x94, 0x23, 0xab, 0x1a, 0x03, 0x75,
            0x89, 0x57, 0x86, 0xc4, 0xbb, 0x46, 0xe9, 0x56,
            0x5f, 0xde, 0x0b, 0x53, 0x44, 0x76, 0x67, 0x40,
            0xaf, 0x26, 0x8a, 0xdb, 0x32, 0x32, 0x2e, 0x5c
        ])),
        u=0x0d,
        v=bytearray_to_int(bytearray([
            0x60, 0xca, 0x1e, 0x32, 0xaa, 0x47, 0x5b, 0x34,
            0x84, 0x88, 0xc3, 0x8f, 0xab, 0x07, 0x64, 0x9c,
            0xe7, 0xef, 0x8d, 0xbe, 0x87, 0xf2, 0x2e, 0x81,
            0xf9, 0x2b, 0x25, 0x92, 0xdb, 0xa3, 0x00, 0xe7
        ])),
    ),
    'id-tc26-gost-3410-2012-512-paramSetC': dict(
        p=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
            0xc7
        ])),
        a=bytearray_to_int(bytearray([
            0x00, 0xdc, 0x92, 0x03, 0xe5, 0x14, 0xa7, 0x21,
            0x87, 0x54, 0x85, 0xa5, 0x29, 0xd2, 0xc7, 0x22,
            0xfb, 0x18, 0x7b, 0xc8, 0x98, 0x0e, 0xb8, 0x66,
            0x64, 0x4d, 0xe4, 0x1c, 0x68, 0xe1, 0x43, 0x06,
            0x45, 0x46, 0xe8, 0x61, 0xc0, 0xe2, 0xc9, 0xed,
            0xd9, 0x2a, 0xde, 0x71, 0xf4, 0x6f, 0xcf, 0x50,
            0xff, 0x2a, 0xd9, 0x7f, 0x95, 0x1f, 0xda, 0x9f,
            0x2a, 0x2e, 0xb6, 0x54, 0x6f, 0x39, 0x68, 0x9b,
            0xd3
        ])),
        b=bytearray_to_int(bytearray([
            0x00, 0xb4, 0xc4, 0xee, 0x28, 0xce, 0xbc, 0x6c,
            0x2c, 0x8a, 0xc1, 0x29, 0x52, 0xcf, 0x37, 0xf1,
            0x6a, 0xc7, 0xef, 0xb6, 0xa9, 0xf6, 0x9f, 0x4b,
            0x57, 0xff, 0xda, 0x2e, 0x4f, 0x0d, 0xe5, 0xad,
            0xe0, 0x38, 0xcb, 0xc2, 0xff, 0xf7, 0x19, 0xd2,
            0xc1, 0x8d, 0xe0, 0x28, 0x4b, 0x8b, 0xfe, 0xf3,
            0xb5, 0x2b, 0x8c, 0xc7, 0xa5, 0xf5, 0xbf, 0x0a,
            0x3c, 0x8d, 0x23, 0x19, 0xa5, 0x31, 0x25, 0x57,
            0xe1
        ])),
        e=0x01,
        d=bytearray_to_int(bytearray([
            0x00, 0x9e, 0x4f, 0x5d, 0x8c, 0x01, 0x7d, 0x8d,
            0x9f, 0x13, 0xa5, 0xcf, 0x3c, 0xdf, 0x5b, 0xfe,
            0x4d, 0xab, 0x40, 0x2d, 0x54, 0x19, 0x8e, 0x31,
            0xeb, 0xde, 0x28, 0xa0, 0x62, 0x10, 0x50, 0x43,
            0x9c, 0xa6, 0xb3, 0x9e, 0x0a, 0x51, 0x5c, 0x06,
            0xb3, 0x04, 0xe2, 0xce, 0x43, 0xe7, 0x9e, 0x36,
            0x9e, 0x91, 0xa0, 0xcf, 0xc2, 0xbc, 0x2a, 0x22,
            0xb4, 0xca, 0x30, 0x2d, 0xbb, 0x33, 0xee, 0x75,
            0x50
        ])),
        m=bytearray_to_int(bytearray([
            0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0x26, 0x33, 0x6e, 0x91, 0x94, 0x1a, 0xac,
            0x01, 0x30, 0xce, 0xa7, 0xfd, 0x45, 0x1d, 0x40,
            0xb3, 0x23, 0xb6, 0xa7, 0x9e, 0x9d, 0xa6, 0x84,
            0x9a, 0x51, 0x88, 0xf3, 0xbd, 0x1f, 0xc0, 0x8f,
            0xb4
        ])),
        q=bytearray_to_int(bytearray([
            0x3f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
            0xc9, 0x8c, 0xdb, 0xa4, 0x65, 0x06, 0xab, 0x00,
            0x4c, 0x33, 0xa9, 0xff, 0x51, 0x47, 0x50, 0x2c,
            0xc8, 0xed, 0xa9, 0xe7, 0xa7, 0x69, 0xa1, 0x26,
            0x94, 0x62, 0x3c, 0xef, 0x47, 0xf0, 0x23, 0xed
        ])),
        x=bytearray_to_int(bytearray([
            0x00, 0xe2, 0xe3, 0x1e, 0xdf, 0xc2, 0x3d, 0xe7,
            0xbd, 0xeb, 0xe2, 0x41, 0xce, 0x59, 0x3e, 0xf5,
            0xde, 0x22, 0x95, 0xb7, 0xa9, 0xcb, 0xae, 0xf0,
            0x21, 0xd3, 0x85, 0xf7, 0x07, 0x4c, 0xea, 0x04,
            0x3a, 0xa2, 0x72, 0x72, 0xa7, 0xae, 0x60, 0x2b,
            0xf2, 0xa7, 0xb9, 0x03, 0x3d, 0xb9, 0xed, 0x36,
            0x10, 0xc6, 0xfb, 0x85, 0x48, 0x7e, 0xae, 0x97,
            0xaa, 0xc5, 0xbc, 0x79, 0x28, 0xc1, 0x95, 0x01,
            0x48
        ])),
        y=bytearray_to_int(bytearray([
            0x00, 0xf5, 0xce, 0x40, 0xd9, 0x5b, 0x5e, 0xb8,
            0x99, 0xab, 0xbc, 0xcf, 0xf5, 0x91, 0x1c, 0xb8,
            0x57, 0x79, 0x39, 0x80, 0x4d, 0x65, 0x27, 0x37,
            0x8b, 0x8c, 0x10, 0x8c, 0x3d, 0x20, 0x90, 0xff,
            0x9b, 0xe1, 0x8e, 0x2d, 0x33, 0xe3, 0x02, 0x1e,
            0xd2, 0xef, 0x32, 0xd8, 0x58, 0x22, 0x42, 0x3b,
            0x63, 0x04, 0xf7, 0x26, 0xaa, 0x85, 0x4b, 0xae,
            0x07, 0xd0, 0x39, 0x6e, 0x9a, 0x9a, 0xdd, 0xc4,
            0x0f
        ])),
        u=0x12,
        v=bytearray_to_int(bytearray([
            0x46, 0x9a, 0xf7, 0x9d, 0x1f, 0xb1, 0xf5, 0xe1,
            0x6b, 0x99, 0x59, 0x2b, 0x77, 0xa0, 0x1e, 0x2a,
            0x0f, 0xdf, 0xb0, 0xd0, 0x17, 0x94, 0x36, 0x8d,
            0x9a, 0x56, 0x11, 0x7f, 0x7b, 0x38, 0x66, 0x95,
            0x22, 0xdd, 0x4b, 0x65, 0x0c, 0xf7, 0x89, 0xee,
            0xbf, 0x06, 0x8c, 0x5d, 0x13, 0x97, 0x32, 0xf0,
            0x90, 0x56, 0x22, 0xc0, 0x4b, 0x2b, 0xaa, 0xe7,
            0x60, 0x03, 0x03, 0xee, 0x73, 0x00, 0x1a, 0x3d
        ])),
    ),
}

def new(mode, curve):
    """Creates a new signature object and returns it.

       Args:
          :mode: Signature generation or verification mode.
          :curve: Parameters of the elliptic curve.

       Return:
          New signature object.

       Exception:
          ValueError('unsupported signature mode') - in case of unsupported signature mode.
          ValueError('invalid parameters of the elliptic curve') - if the elliptic curve
             parameters are incorrect.
    """
    is_edvards_and_canonical = (
        'e' in curve and 'd' in curve and 'u' in curve and 'v' in curve and
        'x' in curve and 'y' in curve and 'a' in curve and 'b' in curve)
    is_edvards_only = (
        'e' in curve and 'd' in curve and 'u' in curve and 'v' in curve and
        'x' not in curve and 'y' not in curve and 'a' not in curve and 'b' not in curve)
    is_canonical_only = (
        'e' not in curve and 'd' not in curve and 'u' not in curve and 'v' not in curve and
        'x' in curve and 'y' in curve and 'a' in curve and 'b' in curve)
    try:
        if is_edvards_and_canonical:
            result = GOST34102012(mode=mode,
                                  p=curve['p'],
                                  a=curve['a'],
                                  b=curve['b'],
                                  e=curve['e'],
                                  d=curve['d'],
                                  m=curve['m'],
                                  q=curve['q'],
                                  x=curve['x'],
                                  y=curve['y'],
                                  u=curve['u'],
                                  v=curve['v'])
        elif is_edvards_only:
            result = GOST34102012(mode=mode,
                                  p=curve['p'],
                                  a=None,
                                  b=None,
                                  e=curve['e'],
                                  d=curve['d'],
                                  m=curve['m'],
                                  q=curve['q'],
                                  x=None,
                                  y=None,
                                  u=curve['u'],
                                  v=curve['v'])
        elif is_canonical_only:
            result = GOST34102012(mode=mode,
                                  p=curve['p'],
                                  a=curve['a'],
                                  b=curve['b'],
                                  e=None,
                                  d=None,
                                  m=curve['m'],
                                  q=curve['q'],
                                  x=curve['x'],
                                  y=curve['y'],
                                  u=None,
                                  v=None)
    except ValueError as err:
        print(err)
        sys_exit()
    return result

class GOST34102012:
    """Сlass that implements processes for creating and verifying an electronic digital
       signature with GOST 34.10-2012.

       Methods:
          :sign(): creating a signature.
          :verify(): signature verification.
          :public_key_generate(): generating a public key.

    """
    # pylint: disable=too-many-instance-attributes
    def __init__(self, mode, **curve_param):
        #Initialize the signature object
        if mode not in (MODE_256, MODE_512):
            raise ValueError('ValueError: unsupported signature mode')
        self._p = curve_param['p']
        self._a = curve_param['a']
        self._b = curve_param['b']
        self._e = curve_param['e']
        self._d = curve_param['d']
        self._m = curve_param['m']
        self._q = curve_param['q']
        self._x = curve_param['x']
        self._y = curve_param['y']
        self._u = curve_param['u']
        self._v = curve_param['v']
        if self._a is None or self._b is None or self._x is None or self._y is None:
            self._edvards_to_canonical()
        if self._m == self._p:
            raise ValueError('ValueError: invalid parameters of the elliptic curve')
        if mode == MODE_256:
            for i in range(1, 32):
                if self._p ** i % self._q == 1 % self._q:
                    raise ValueError('ValueError: invalid parameters of the elliptic curve')
            if self._q < 2 ** 254 or self._q > 2 ** 256:
                raise ValueError('ValueError: invalid parameters of the elliptic curve')
            self._size = 32
        elif mode == MODE_512:
            for i in range(1, 132):
                if self._p ** 1 % self._q == 1 % self._q:
                    raise ValueError('ValueError: invalid parameters of the elliptic curve')
            if self._q < 2 ** 508 or self._q > 2 ** 512:
                raise ValueError('ValueError: invalid parameters of the elliptic curve')
            self._size = 64
        right_side_equation = self._y * self._y % self._p
        left_side_equation = (self._x ** 3 + self._x * self._a + self._b) % self._p
        if right_side_equation != left_side_equation:
            raise ValueError('ValueError: invalid parameters of the elliptic curve')
    # pylint: enable=too-many-instance-attributes

    @staticmethod
    def _invert(value, n_mod):
        #Function for finding the inverse value in the residue
        #ring (based on the extended Euclid algorithm)
        def gcdex(value, n_mod):
            if n_mod == 0:
                gcd_result = value, 1, 0
            else:
                gcd, x_value, y_value = gcdex(n_mod, value % n_mod)
                gcd_result = gcd, y_value, x_value - y_value * (value // n_mod)
            return gcd_result
        result = gcdex(value, n_mod)[1]
        if result < 0:
            result = result + n_mod
        return result

    def _add(self, x_1, x_2, y_1, y_2):
        #Function for adding two points on a curve
        compare_x = compare(int_to_bytearray(x_1, self._size), int_to_bytearray(x_2, self._size))
        compare_y = compare(int_to_bytearray(y_1, self._size), int_to_bytearray(y_2, self._size))
        if compare_x and compare_y:
            grad = (3 * x_1 * x_1 + self._a) * self._invert(2 * y_1, self._p) % self._p
        else:
            d_x = (x_2 - x_1) % self._p
            d_y = (y_2 - y_1) % self._p
            grad = (d_y * self._invert(d_x, self._p)) % self._p
        x_3 = (grad * grad - x_1 - x_2) % self._p
        y_3 = (grad * (x_1 - x_3) - y_1) % self._p
        return x_3, y_3

    @staticmethod
    def _bits(value):
        #Function for getting the bit representation of a number
        while value:
            yield value & 1
            value = value >> 1

    def _mul_point(self, mul_value, x_op=None, y_op=None):
        #Function for calculating a multiple point of an elliptic curve
        mul_value = mul_value - 1
        if x_op or y_op:
            x_prev = x_op
            y_prev = y_op
        else:
            x_prev = self._x
            y_prev = self._y
        x_next = x_prev
        y_next = y_prev
        for bit in self._bits(mul_value):
            if bit == 1:
                x_next, y_next = self._add(x_next, x_prev, y_next, y_prev)
            x_prev, y_prev = self._add(x_prev, x_prev, y_prev, y_prev)
        return x_next, y_next

    def _edvards_to_canonical(self):
        #Translation function to canonical representation of an elliptic curve from the
        #representation as twisted Edwards curves
        ed_s = (self._e - self._d) * self._invert(4, self._p) % self._p
        ed_t = (self._e + self._d) * self._invert(6, self._p) % self._p
        self._a = (ed_s ** 2 - 3 * ed_t ** 2) % self._p
        self._b = (2 * ed_t ** 3 - ed_t * ed_s ** 2) % self._p
        self._x = ((ed_s * (1 + self._v)) * self._invert(1 - self._v, self._p) + ed_t)\
            % self._p
        self._y = ((ed_s * (1 + self._v)) * self._invert((1 - self._v) * self._u, self._p))\
            % self._p

    def sign(self, private_key, digest, rand_k=None):
        """Creating a signature.

           Args:
              :private_key: Private signature key (as a byte object).
              :digest: Digest for which the signature is calculated.
              :rand_k: Random (pseudo-random) number (as a byte object). By default, it is
                 generated by the function itself.

           Return:
              Signature for provided digest (as a byte object).

           Exception:
              ValueError('invalid random value size') - in case of invalid 'rand_k' size.
        """
        if not isinstance(private_key, (bytes, bytearray)):
            raise ValueError('ValueError: invalid private key value')
        if len(private_key) != self._size:
            raise ValueError('ValueError: invalid private key size')
        sign_e = bytearray_to_int(digest) % self._q
        if compare_to_zero(int_to_bytearray(sign_e, self._size)):
            sign_e = 1
        sign_r = 0
        sign_s = 0
        sign_k = 0
        while compare_to_zero(int_to_bytearray(sign_s, self._size)):
            while compare_to_zero(int_to_bytearray(sign_r, self._size)):
                if rand_k is None:
                    sign_rand_k = os.urandom(self._size)
                    while bytearray_to_int(sign_rand_k) >= self._q:
                        sign_rand_k = os.urandom(self._size)
                else:
                    if len(rand_k) != self._size:
                        private_key = zero_fill(len(private_key))
                        raise ValueError('ValueError: invalid random value size')
                    sign_rand_k = rand_k
                sign_k = bytearray_to_int(sign_rand_k)
                sign_c = self._mul_point(sign_k)
                sign_r = sign_c[0] % self._q
            sign_s = (sign_r * bytearray_to_int(private_key) + sign_k * sign_e) % self._q
        sign_r = int_to_bytearray(sign_r, self._size)
        sign_s = int_to_bytearray(sign_s, self._size)
        private_key = zero_fill(len(private_key))
        return sign_r + sign_s

    def verify(self, public_key, digest, signature):
        """Verify a signature.

           Args:
              :public_key: Public signature key (as a byte object).
              :digest: Digest for which to be checked signature (as a byte object).
              :signature: Signature of the digest being checked (as a byte object).

           Return:
              The result of the signature verification ('True' or 'False').

           Exception:
              ValueError('Invalid signature size') - if the signature size is incorrect.
        """
        if len(signature) != self._size * 2:
            raise ValueError('ValueError: invalid signature size')
        public_key = bytearray_to_int(public_key[:self._size]),\
                     bytearray_to_int(public_key[self._size:])
        sign_r = bytearray_to_int(signature[:self._size])
        sign_s = bytearray_to_int(signature[self._size:])
        if sign_r <= 0 or sign_r >= self._q or sign_s <= 0 or sign_s >= self._q:
            return False
        sign_e = bytearray_to_int(digest) % self._q
        if compare_to_zero(int_to_bytearray(sign_e, self._size)):
            sign_e = 1
        sign_v = self._invert(sign_e, self._q)
        sign_z_1 = sign_s * sign_v % self._q
        sign_z_2 = self._q - sign_r * sign_v % self._q
        sign_p = self._mul_point(sign_z_1)
        sign_q = self._mul_point(sign_z_2, public_key[0], public_key[1])
        sign_c = self._add(sign_p[0], sign_q[0], sign_p[1], sign_q[1])
        sign_r_check = sign_c[0] % self._q
        return compare(int_to_bytearray(sign_r_check, self._size),
                       int_to_bytearray(sign_r, self._size))

    def public_key_generate(self, private_key):
        """Generating a public key

           Args:
              :private_key: Private signature key (as a byte object).

           Return:
              Public key (as a byte object).

           Exception:
              ValueError('ValueError: invalid private key size') - in case of invalid private
                 key size.
              ValueError('ValueError: invalid private key value') - in case of invalid private
                 key value.
        """
        if not isinstance(private_key, (bytes, bytearray)):
            raise ValueError('ValueError: invalid private key value')
        if len(private_key) != self._size:
            raise ValueError('ValueError: invalid private key size')
        private_key = bytearray_to_int(private_key)
        public_key = self._mul_point(private_key)
        public_key_x = int_to_bytearray(public_key[0], self._size)
        public_key_y = int_to_bytearray(public_key[1], self._size)
        private_key = 0
        return public_key_x + public_key_y



"""The module implementing the hash calculation algorithm GOST 34.11-2012 ('Streebog').
"""

__title__ = 'gosthash'
__author__ = 'Evgeny Drobotun'
__author_email__ = 'drobotun@xakep.ru'
__license__ = 'MIT'
__copyright__ = 'Copyright (C) 2020 Evgeny Drobotun'

from .gost_34_11_2012 import *
